package com.telpo.dipperposition.config.db;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.data.mongodb.MongoDbFactory;
import org.springframework.data.mongodb.core.SimpleMongoClientDbFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @program: DataPushServer
 * @description: mongdb数据库连接上下文
 * @author: linwl
 * @create: 2020-07-11 14:31
 */
@Component
public class MongoDbContext {

  private static final Map<String, MongoDbFactory> MONGO_CLIENT_DB_FACTORY_MAP = new HashMap<>();
  private static final ThreadLocal<MongoDbFactory> MONGO_DB_FACTORY_THREAD_LOCAL = new ThreadLocal<>();
  @Autowired
  MongoListProperties mongoListProperties;

  public static MongoDbFactory getMongoDbFactory() {
    return MONGO_DB_FACTORY_THREAD_LOCAL.get();
  }

  public static void setMongoDbFactory(String name) {
    MONGO_DB_FACTORY_THREAD_LOCAL.set(MONGO_CLIENT_DB_FACTORY_MAP.get(name));
  }

  public static void removeMongoDbFactory() {
    MONGO_DB_FACTORY_THREAD_LOCAL.remove();
  }

  @PostConstruct
  public void afterPropertiesSet() {
    if (!CollectionUtils.isEmpty(mongoListProperties.getDblist())) {
      mongoListProperties
          .getDblist()
          .forEach(
              info -> {
                MONGO_CLIENT_DB_FACTORY_MAP.put(
                    info.getDatabase(), new SimpleMongoClientDbFactory(info.getUri()));
              });
    }
  }

  @Bean(name = "mongoTemplate")
  public MultiMongoTemplate dynamicMongoTemplate() {
    Iterator<MongoDbFactory> iterator = MONGO_CLIENT_DB_FACTORY_MAP.values().iterator();
    return new MultiMongoTemplate(iterator.next());
  }

  @Bean(name = "mongoDbFactory")
  public MongoDbFactory mongoDbFactory() {
    Iterator<MongoDbFactory> iterator = MONGO_CLIENT_DB_FACTORY_MAP.values().iterator();
    return iterator.next();
  }
}
